from unittest import mock
from unittest.mock import patch

import pandas as pd
import pytest
import synapseclient

from genie_registry.fusions import fusions


def createMockTable(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return(table)


def table_query_results(*args):
    return(table_query_results_map[args])


databaseMapping = pd.DataFrame(dict(Database=['bed'],
                                    Id=['syn8457748']))
symbols = pd.DataFrame(dict(Hugo_Symbol=['AAK1', 'AAED1', 'AAAS'],
                            ID=['AAK1', 'AAED', 'AAAS']))

# This is the gene positions that all bed dataframe will be processed against
table_query_results_map = {
    ("SELECT * FROM syn10967259",): createMockTable(databaseMapping),
    ("select Hugo_Symbol, ID from syn8457748 where CENTER = 'SAGE'",):
        createMockTable(symbols),
}
ENTITY = synapseclient.Project("testing",
                               annotations={'dbMapping': ["syn10967259"]})

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results
fusionClass = fusions(syn, "SAGE")


def test_processing():
    expectedFusionDf = pd.DataFrame({
        "HUGO_SYMBOL": ['AAED1', 'AAK1', 'AAAS5'],
        "ENTREZ_GENE_ID": [0, 0, 0],
        "CENTER": ["SAGE", "SAGE", "SAGE"],
        "TUMOR_SAMPLE_BARCODE": ["GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1",
                                 "GENIE-SAGE-ID1-3"],
        "FUSION": ["AAED1-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
        "DNA_SUPPORT": ["foo", 'foo', 'foo'],
        "RNA_SUPPORT": ["foo", 'foo', 'foo'],
        "METHOD": ["foo", 'foo', 'foo'],
        "FRAME": ["foo", 'foo', 'foo'],
        "ID": ['AAED', 'AAK1', 'AAAS5']})

    fusionDf = pd.DataFrame({
        "HUGO_SYMBOL": ['AAED', 'AAK1', 'AAAS5'],
        "ENTREZ_GENE_ID": [0, 0, float('nan')],
        "CENTER": ["SAGE", "SAGE", "SAGE"],
        "TUMOR_SAMPLE_BARCODE": ["ID1-1", "ID2-1", "ID1-3"],
        "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
        "DNA_SUPPORT": ["foo", 'foo', 'foo'],
        "RNA_SUPPORT": ["foo", 'foo', 'foo'],
        "METHOD": ["foo", 'foo', 'foo'],
        "FRAME": ["foo", 'foo', 'foo']})

    newFusionDf = fusionClass._process(fusionDf, databaseMapping)
    assert expectedFusionDf.equals(newFusionDf[expectedFusionDf.columns])


def test_validation():
    with pytest.raises(AssertionError):
        fusionClass.validateFilename(['foo'])
    assert fusionClass.validateFilename(["data_fusions_SAGE.txt"]) == "fusions"

    fusionDf = pd.DataFrame({
        "HUGO_SYMBOL": ['AAED', 'AAK1', 'AAAS'],
        "ENTREZ_GENE_ID": [0, 0, 0],
        "CENTER": ["SAGE", "SAGE", "SAGE"],
        "TUMOR_SAMPLE_BARCODE": ["ID1-1", "ID2-1", "ID1-3"],
        "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
        "DNA_SUPPORT": ["foo", 'foo', 'foo'],
        "RNA_SUPPORT": ["foo", 'foo', 'foo'],
        "METHOD": ["foo", 'foo', 'foo'],
        "FRAME": ["foo", 'foo', 'foo']})
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, False, "syn1234")
        assert error == ""
        assert warning == ""

    fusionDf = pd.DataFrame({
        "HUGO_SYMBOL": [float('nan'), 'AAK1', 'AAAS'],
        "CENTER": ["SAGE", "SAGE", "SAGE"],
        "TUMOR_SAMPLE_BARCODE": ["ID1-1", "ID2-1", "ID1-3"],
        "FUSION": ["AAED-AAK1", "AAAS-AAK1", "AAAS-AAK1"],
        "DNA_SUPPORT": ["foo", 'foo', 'foo'],
        "RNA_SUPPORT": ["foo", 'foo', 'foo'],
        "METHOD": ["foo", 'foo', 'foo'],
        "FRAME": ["foo", 'foo', 'foo']})
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, False, "syn1234")
        expectedErrors = (
            "Your fusion file must at least have these headers: ENTREZ_GENE_ID.\n"
            "Your fusion file should not have any NA/blank Hugo Symbols.\n")

        assert error == expectedErrors
        assert warning == ""
    # Do not check symbols
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = fusionClass._validate(fusionDf, True, "syn1234")
        expectedErrors = (
            "Your fusion file must at least have these headers: ENTREZ_GENE_ID.\n")
        assert error == expectedErrors
        assert warning == ""



from unittest import mock
from unittest.mock import patch
import pytest

import pandas as pd
import synapseclient

from genie_registry.cna import cna


def createMockTable(dataframe):
    table = mock.create_autospec(synapseclient.table.CsvFileTable)
    table.asDataFrame.return_value = dataframe
    return(table)


def table_query_results(*args):
    return(table_query_results_map[args])


database_mapping = pd.DataFrame(dict(
    Database=['bed'],
    Id=['syn8457748']))
symbols = pd.DataFrame(dict(
    Hugo_Symbol=['AAK1', 'AAED1', 'AAAS', 'AAED1'],
    ID=['AAK1', 'AAED', 'AAAS', 'AAD']))
# This is the gene positions that all bed dataframe will be processed against
table_query_results_map = {
    ("SELECT * FROM syn10967259",): createMockTable(database_mapping),
    ("select Hugo_Symbol, ID from syn8457748 where CENTER = 'SAGE'",):
        createMockTable(symbols), }

syn = mock.create_autospec(synapseclient.Synapse)
syn.tableQuery.side_effect = table_query_results

ENTITY = synapseclient.Project("testing",
                               annotations={'dbMapping': ["syn10967259"]})
cna_class = cna(syn, "SAGE")


def test_processing():

    order = ["Hugo_Symbol", "Entrez_gene_id", "Id1-1", "Id2-1"]

    expected_cnadf = pd.DataFrame({
        "Hugo_Symbol": ['AAED1', 'AAK1', 'AAAS'],
        "GENIE-SAGE-Id1-1": [-0.5, 2.0, 0.5],
        "GENIE-SAGE-Id2-1": [1.0, 1.5, -1.5]})

    cnadf = pd.DataFrame({
        "Hugo_Symbol": ['AAED', 'AAK1', 'AAAS'],
        "Entrez_gene_id": [0, 0, 0],
        "Id1-1": [-0.5, 2, 0.5],
        "Id2-1": [1, 1.5, -1.5]})
    cnadf = cnadf[order]
    new_cnadf = cna_class._process(cnadf, database_mapping)
    assert expected_cnadf.equals(new_cnadf[expected_cnadf.columns])

    order = ["Hugo_Symbol", "Entrez_gene_id", "GENIE-SAGE-Id1-1",
             "GENIE-SAGE-Id2-1"]

    expectedCnaDf = pd.DataFrame({
        "Hugo_Symbol": ['AAAS', 'AAED1'],
        "GENIE-SAGE-Id1-1": [float('nan'), 1.0],
        "GENIE-SAGE-Id2-1": [float('nan'), 2.0]})

    cnaDf = pd.DataFrame({
        "Hugo_Symbol": ['AAED', 'AAED1', 'foo', 'AAAS', 'AAD'],
        "Entrez_gene_id": [0, 0, 0, 0, 0],
        "GENIE-SAGE-Id1-1": [1, 1, 0, float('nan'), 1],
        "GENIE-SAGE-Id2-1": [2, 0, -1, float('nan'), float('nan')]})
    cnaDf = cnaDf[order]
    newCnaDf = cna_class._process(cnaDf, database_mapping)
    newCnaDf.reset_index(inplace=True, drop=True)
    pd.testing.assert_frame_equal(
        expectedCnaDf, newCnaDf[expectedCnaDf.columns])


def test_validation():
    with pytest.raises(AssertionError):
        cna_class.validateFilename(["foo"])
    assert cna_class.validateFilename(["data_CNA_SAGE.txt"]) == "cna"

    order = ["Hugo_Symbol", "Entrez_gene_id",
             "GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1"]
    cnaDf = pd.DataFrame({
        "Hugo_Symbol": ['AAED', 'AAK1', 'AAAS'],
        "Entrez_gene_id": [0, 0, 0],
        "GENIE-SAGE-ID1-1": [-2, 0, 1],
        "GENIE-SAGE-ID2-1": [0.5, 1.5, -1.5],
        "GENIE-SAGE-ID3-1": [float('nan'), 2, 'NA']})
    cnaDf = cnaDf[order]
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False, "syn1234")
        assert error == ""
        assert warning == ""

    cnaDf = pd.DataFrame(
        {"Hugo_Symbol": ['foo', 'AAED', 'AAED1', 'AAD'],
         "GENIE-SAGE-ID1-1": [1, 2, float('nan'), 1],
         "GENIE-SAGE-ID2-1": [2, 1, -1, 2]})
    cnaDf.sort_values("Hugo_Symbol", inplace=True)
    cnaDf = cnaDf[["GENIE-SAGE-ID1-1", "Hugo_Symbol", "GENIE-SAGE-ID2-1"]]

    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False, "syn1234")
        expectedErrors = (
            "Your cnv file's first column must be Hugo_Symbol\n"
            "Your CNA file has duplicated Hugo_Symbols (After "
            "remapping of genes): AAD,AAED,AAED1 -> AAED1,AAED1,AAED1.\n")

    assert error == expectedErrors
    assert warning == ""
    order = ["Hugo_Symbol", "GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1"]

    cnaDf = pd.DataFrame({
        "Hugo_Symbol": ['AAK1', 'AAAS', 'AAED1'],
        "GENIE-SAGE-ID1-1": [1, 2, 1],
        "GENIE-SAGE-ID2-1": [2, 1, 3]})
    cnaDf = cnaDf[order]
    with patch.object(syn, "get", return_value=ENTITY):
        error, warning = cna_class._validate(cnaDf, False, "syn1234")
        expectedErrors = (
            "All values must be NA/blank, -2, -1.5, -1, -0.5, 0, "
            "0.5, 1, 1.5, or 2.\n")
        assert error == expectedErrors
        assert warning == ""



from unittest import mock

import pandas as pd
import pytest
import synapseclient

from genie_registry.sampleRetraction import sampleRetraction
from genie_registry.patientRetraction import patientRetraction

syn = mock.create_autospec(synapseclient.Synapse)

sr = sampleRetraction(syn, "SAGE")
pr = patientRetraction(syn, "SAGE")


def test_processing():

    expectedsrDf = pd.DataFrame(dict(
        genieSampleId=["GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1",
                       "GENIE-SAGE-ID3-1", "GENIE-SAGE-ID4-1",
                       "GENIE-SAGE-ID5-1"],
        retractionDate=[1523039400000, 1523039400000, 1523039400000,
                        1523039400000, 1523039400000],
        center=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"]))

    srDf = pd.DataFrame({0: ["ID1-1", "ID2-1", "ID3-1", "ID4-1", "ID5-1"]})

    newsrDf = sr._process(srDf, "2018-04-06T18:30:00")
    assert expectedsrDf.equals(newsrDf[expectedsrDf.columns])

    expectedprDf = pd.DataFrame(dict(
        geniePatientId=["GENIE-SAGE-ID1", "GENIE-SAGE-ID2", "GENIE-SAGE-ID3",
                        "GENIE-SAGE-ID4", "GENIE-SAGE-ID5"],
        retractionDate=[1523125800000, 1523125800000, 1523125800000,
                        1523125800000, 1523125800000],
        center=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"]))

    prDf = pd.DataFrame({0: ["ID1", "ID2", "ID3", "ID4", "ID5"]})

    newprDf = pr._process(prDf, "2018-04-07T18:30:00")
    assert expectedprDf.equals(newprDf[expectedprDf.columns])


def test_validation():
    with pytest.raises(AssertionError):
        sr.validateFilename(["foo"])
    assert sr.validateFilename(["sampleRetraction.csv"]) == "sampleRetraction"

    with pytest.raises(AssertionError):
        pr.validateFilename(["foo"])
    assert pr.validateFilename(
        ["patientRetraction.csv"]) == "patientRetraction"



from functools import partial
import os
import logging

import pandas as pd

from genie.example_filetype_format import FileTypeFormat
from genie import process_functions

logger = logging.getLogger(__name__)


def validateSymbol(x, bedDf, returnMappedDf=True):
    valid=False
    gene = x['HUGO_SYMBOL']
    if sum(bedDf['Hugo_Symbol'] == gene) > 0:
        valid=True
    elif sum(bedDf['ID'] == gene) > 0:
        mismatch = bedDf[bedDf['ID'] == gene]
        mismatch.drop_duplicates(inplace=True)
        logger.info("%s will be remapped to %s" % (gene, mismatch['Hugo_Symbol'].values[0]))
        x['HUGO_SYMBOL'] = mismatch['Hugo_Symbol'].values[0]
    #else:
    #    logger.warning("%s cannot be remapped and will not be released. The symbol must exist in your seq assay ids (bed files) and must be mappable to a gene." % gene)
    #    x['HUGO_SYMBOL'] = float('nan')
        #x['FUSION'] = x['FUSION'].replace("%s-" % gene,"%s-" % x['HUGO_SYMBOL'])
        #x['COMMENTS'] = str(x['COMMENTS']).replace("-%s" % gene,"-%s" % str(x['COMMENTS']))
    if returnMappedDf:
        return(x)
    else:
        return(valid)

# Remap fusion's FUSION column
def remapFusion(gene_dict, DF, col):
    nonmapped = []
    total = []
    for each in DF[col]:
        for key in gene_dict:
            value = gene_dict[key]
            if value == False:
                nonmapped.append(key)
            else:
                each = each.replace("%s-" % key, "%s-" % value)
                each = each.replace("-%s fusion" % key, "-%s fusion" % value)
        total.append(each)
    DF[col] = total 
    return(DF, nonmapped)

class fusions(FileTypeFormat):
   
    _fileType = "fusions"

    _process_kwargs = ["newPath", "databaseSynId", "databaseToSynIdMappingDf"]

    _validation_kwargs = ['nosymbol_check', 'project_id']

    #VALIDATE FILENAME
    def _validateFilename(self, filePath):
        assert os.path.basename(filePath[0]) == "data_fusions_%s.txt" % self.center


    def _process(self, fusion, databaseToSynIdMappingDf):
        fusion.columns = [col.upper() for col in fusion.columns]
        fusion['CENTER'] = self.center
        newsamples = [process_functions.checkGenieId(i,self.center) for i in fusion['TUMOR_SAMPLE_BARCODE']]
        fusion['TUMOR_SAMPLE_BARCODE'] = newsamples

        #This is temporary, because comments column will be removed
        #if fusion.get("COMMENTS") is None:
        #    fusion['COMMENTS'] = ""
        # #Will remove comments column
        # fusion['COMMENTS'] = ""
        fusion['ENTREZ_GENE_ID'] = fusion['ENTREZ_GENE_ID'].fillna(0)
        fusion = fusion.drop_duplicates()
        fusion['ID'] = fusion['HUGO_SYMBOL'].copy()
        bedSynId = process_functions.getDatabaseSynId(self.syn, "bed",
                                                      databaseToSynIdMappingDf=databaseToSynIdMappingDf)
        bed = self.syn.tableQuery("select Hugo_Symbol, ID from %s where CENTER = '%s'" % (bedSynId, self.center))
        bedDf = bed.asDataFrame()
        fusion = fusion.apply(lambda x: validateSymbol(x, bedDf), axis=1)
        #Create nonmapped gene dict
        temp = fusion[fusion['HUGO_SYMBOL'] != fusion['ID']]
        foo = temp[~temp.HUGO_SYMBOL.isnull()]
        temp = foo[['HUGO_SYMBOL','ID']]
        temp.drop_duplicates(inplace=True)
        temp.index=temp.ID
        del temp['ID']
        # fusion = fusion[~fusion['HUGO_SYMBOL'].isnull()]
        fusion['FUSION'] = fusion['FUSION'].fillna("")
        fusion, nonmapped = remapFusion(temp.to_dict()['HUGO_SYMBOL'], fusion, "FUSION")
        #fusion, nonmapped = remapFusion(temp.to_dict()['HUGO_SYMBOL'], fusion, "COMMENTS")
        fusion['ENTREZ_GENE_ID'] = [int(float(i)) for i in fusion['ENTREZ_GENE_ID']]
        return(fusion)

    #PROCESSING
    def process_steps(self, fusion, databaseSynId, newPath, databaseToSynIdMappingDf):
        fusion = self._process(fusion, databaseToSynIdMappingDf)
        process_functions.updateData(self.syn, databaseSynId, fusion, self.center, toDelete=True)
        fusion.to_csv(newPath, sep="\t",index=False)
        return(newPath)

    def _validate(self, fusionDF, nosymbol_check, project_id):
        total_error = ""
        warning = ""


# Frame: "in-frame" or "frameshift".
# Fusion_Status (OPTIONAL): An assessment of the mutation type (i.e., "SOMATIC", "GERMLINE", "UNKNOWN", or empty)

        fusionDF.columns = [col.upper() for col in fusionDF.columns]

        REQUIRED_HEADERS = pd.Series(['HUGO_SYMBOL','ENTREZ_GENE_ID','CENTER','TUMOR_SAMPLE_BARCODE','FUSION','DNA_SUPPORT','RNA_SUPPORT','METHOD','FRAME'])
        if fusionDF.get("COMMENTS") is None:
            fusionDF['COMMENTS'] = float('nan')
        if not all(REQUIRED_HEADERS.isin(fusionDF.columns)):
            total_error += "Your fusion file must at least have these headers: %s.\n" % ",".join(REQUIRED_HEADERS[~REQUIRED_HEADERS.isin(fusionDF.columns)])
        if process_functions.checkColExist(fusionDF, "HUGO_SYMBOL") and not nosymbol_check:
           # logger.info("VALIDATING %s GENE SYMBOLS" % os.path.basename(filePath))
            #invalidated_genes = fusionDF["HUGO_SYMBOL"].drop_duplicates().apply(validateSymbol)
            databaseToSynIdMappingDf = process_functions.get_synid_database_mappingdf(self.syn, project_id)
            bedSynId = process_functions.getDatabaseSynId(self.syn, "bed",
                                                          databaseToSynIdMappingDf=databaseToSynIdMappingDf)
            bed = self.syn.tableQuery("select Hugo_Symbol, ID from %s where CENTER = '%s'" % (bedSynId, self.center))
            bedDf = bed.asDataFrame()
            #invalidated_genes = self.pool.map(process_functions.validateSymbol, fusionDF["HUGO_SYMBOL"].drop_duplicates())
            if fusionDF["HUGO_SYMBOL"].isnull().any():
                total_error += "Your fusion file should not have any NA/blank Hugo Symbols.\n"
            # fusionDF = fusionDF.drop_duplicates("HUGO_SYMBOL").apply(lambda x: validateSymbol(x, bedDf), axis=1)

        # if process_functions.checkColExist(fusionDF, "DNA_SUPPORT"):
        #     if not fusionDF.DNA_SUPPORT.isin(["yes","no","unknown"]).all():
        #         total_error += "Your fusion file's DNA_SUPPORT column must be 'yes', 'no', or 'unknown'"

        # if process_functions.checkColExist(fusionDF, "RNA_SUPPORT"):
        #     if not fusionDF.RNA_SUPPORT.isin(["yes","no","unknown"]).all():
        #         total_error += "Your fusion file's RNA_SUPPORT column must be 'yes', 'no', or 'unknown'"

        # if process_functions.checkColExist(fusionDF, "FRAME"):
        #     if not fusionDF.FRAME.isin(["in-frame","frameshift"]).all():
        #         total_error += "Your fusion file's FRAME column must be 'in-frame', or 'frameshift'"
    

        return(total_error, warning)



from unittest import mock

import pandas as pd
import pytest
import synapseclient

from genie_registry.seg import seg
from genie_registry.cbs import cbs

syn = mock.create_autospec(synapseclient.Synapse)

segClass = seg(syn, "SAGE")
cbsClass = cbs(syn, "SAGE")


def test_processing():
    expectedSegDf = pd.DataFrame({
        "ID": ['GENIE-SAGE-ID1-1', 'GENIE-SAGE-ID2-1', 'GENIE-SAGE-ID3-1',
               'GENIE-SAGE-ID4-1', 'GENIE-SAGE-ID5-1'],
        "CHROM": ['1', '2', '3', '4', '5'],
        "LOCSTART": [1, 2, 3, 4, 3],
        "LOCEND": [1, 2, 3, 4, 2],
        "NUMMARK": [1, 2, 3, 4, 3],
        "SEGMEAN": [1, 2, 3.9, 4, 3],
        "CENTER": ["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"]})

    segDf = pd.DataFrame({
        "ID": ['ID1-1', 'ID2-1', 'ID3-1', 'ID4-1', 'ID5-1'],
        "CHROM": ['chr1', 2, 3, 4, 5],
        "LOC.START": [1, 2, 3, 4, 3],
        "LOC.END": [1, 2, 3, 4, 2],
        "NUM.MARK": [1, 2, 3, 4, 3],
        "SEG.MEAN": [1, 2, 3.9, 4, 3]})

    newSegDf = segClass._process(segDf)
    assert expectedSegDf.equals(newSegDf[expectedSegDf.columns])

    newSegDf = cbsClass._process(segDf)
    assert expectedSegDf.equals(newSegDf[expectedSegDf.columns])


def test_validation():
    with pytest.raises(AssertionError):
        segClass.validateFilename(["foo"])
    assert segClass.validateFilename(["genie_data_cna_hg19_SAGE.seg"]) == "seg"
    assert cbsClass.validateFilename(["genie_data_cna_hg19_SAGE.cbs"]) == "cbs"

    segDf = pd.DataFrame({
        "ID": ['ID1', 'ID2', 'ID3', 'ID4', 'ID5'],
        "CHROM": [1, 2, 3, 4, 5],
        "LOC.START": [1, 2, 3, 4, 3],
        "LOC.END": [1, 2, 3, 4, 3],
        "NUM.MARK": [1, 2, 3, 4, 3],
        "SEG.MEAN": [1, 2, 3, 4, 3]})

    error, warning = segClass._validate(segDf)
    assert error == ""
    assert warning == ""

    segDf = pd.DataFrame({
        "ID": ['ID1', 'ID2', 'ID3', 'ID4', 'ID5'],
        "CHROM": [1, 2, float('nan'), 4, 5],
        "LOC.START": [1, 2, 3, 4, float('nan')],
        "LOC.END": [1, 2, 3, float('nan'), 3],
        "NUM.MARK": [1, 2, 3, 4, 3]})
    expectedErrors = (
        "Your seg file is missing these headers: SEG.MEAN.\n"
        "Seg: No null or empty values allowed in column(s): "
        "CHROM, LOC.END, LOC.START.\n")
    error, warning = segClass._validate(segDf)
    assert error == expectedErrors
    assert warning == ""

    error, warning = cbsClass._validate(segDf)
    assert error == expectedErrors
    assert warning == ""

    segDf = pd.DataFrame({
        "ID": ['ID1', 'ID2', 'ID3', 'ID4', 'ID5'],
        "CHROM": [1, 2, 3, 4, 5],
        "LOC.START": [1, 2, 3, 4.3, 3],
        "LOC.END": [1, 2, 3.4, 4, 3],
        "NUM.MARK": [1, 2, 3, 33.3, 3],
        "SEG.MEAN": [1, 2, 'f.d', 4, 3]})
    error, warning = segClass._validate(segDf)
    expectedErrors = (
        "Seg: Only integars allowed in these column(s): "
        "LOC.END, LOC.START, NUM.MARK.\n"
        "Seg: Only numerical values allowed in SEG.MEAN.\n")
    assert error == expectedErrors
    assert warning == ""



"""Converts consortium release files to public release files"""
#! /usr/bin/env python3

import logging
import os
import shutil

import synapseclient
import synapseutils
import pandas as pd

from . import process_functions
from . import database_to_staging
from . import create_case_lists

logger = logging.getLogger(__name__)


def storeFile(syn, filePath, parentId, genie_version, name=None):
    """Stores file with genie version as comment

    Args:
        syn: Synapse object
        filePath: Path to file
        parentId: Synapse id of folder

    Returns:
        Stored Entity
    """
    if name is None:
        name = os.path.basename(filePath)
    file_ent = synapseclient.File(filePath, name=name, parent=parentId,
                                  versionComment=genie_version)
    file_ent = syn.store(file_ent)
    return file_ent


def commonVariantFilter(mafDf):
    '''
    This filter returns variants to keep

    Args:
        mafDf: Maf dataframe
    '''
    mafDf['FILTER'] = mafDf['FILTER'].fillna("")
    to_keep = ["common_variant" not in i for i in mafDf['FILTER']]
    mafDf = mafDf[to_keep]
    return mafDf


def consortiumToPublic(syn, processingDate, genie_version,
                       releaseId, databaseSynIdMappingDf,
                       publicReleaseCutOff=365):
    cna_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        "data_CNA_%s.txt" % genie_version)
    clinical_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_clinical_%s.txt' % genie_version)
    clinical_sample_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_clinical_sample_%s.txt' % genie_version)
    clinicl_patient_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_clinical_patient_%s.txt' % genie_version)
    data_gene_panel_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_gene_matrix_%s.txt' % genie_version)
    mutations_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_mutations_extended_%s.txt' % genie_version)
    fusions_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'data_fusions_%s.txt' % genie_version)
    seg_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'genie_public_data_cna_hg19_%s.seg' % genie_version)
    combined_bed_path = os.path.join(
        database_to_staging.GENIE_RELEASE_DIR,
        'genie_combined_%s.bed' % genie_version)

    if not os.path.exists(database_to_staging.GENIE_RELEASE_DIR):
        os.mkdir(database_to_staging.GENIE_RELEASE_DIR)
    if not os.path.exists(database_to_staging.CASE_LIST_PATH):
        os.mkdir(database_to_staging.CASE_LIST_PATH)

    # public release preview
    public_release_preview = databaseSynIdMappingDf['Id'][
        databaseSynIdMappingDf['Database'] == 'public'].values[0]
    public_release_preview_caselist = \
        database_to_staging.find_caselistid(syn, public_release_preview)

    #######################################################################
    # Sponsored projects filter
    #######################################################################
    # if before release date -> go into staging consortium
    # if after date -> go into public
    # sponsoredReleaseDate = syn.tableQuery('SELECT * FROM syn8545108')
    # sponsoredReleaseDateDf = sponsoredReleaseDate.asDataFrame()
    # sponsoredProjectSamples = syn.tableQuery('SELECT * FROM syn8545106')
    # sponsoredProjectSamplesDf = sponsoredProjectSamples.asDataFrame()
    # sponsoredProjectsDf = sponsoredProjectSamplesDf.merge(
    #     sponsoredReleaseDateDf, left_on="sponsoredProject",
    #     right_on="sponsoredProjects")
    # dates = sponsoredProjectsDf['releaseDate'].apply(
    #     lambda date: datetime.datetime.strptime(date, '%b-%Y'))
    # publicReleaseSamples = sponsoredProjectsDf['genieSampleId'][
    #     dates < processingDate]
    #######################################################################

    # SEQ_DATE filter
    # Jun-2015, given processing date (today) -> public release
    # (processing date - Jun-2015 > 12 months)
    consortiumReleaseWalk = synapseutils.walk(syn, releaseId)

    consortiumRelease = next(consortiumReleaseWalk)
    clinical = [syn.get(synid, followLink=True)
                for filename, synid in consortiumRelease[2]
                if filename == "data_clinical.txt"][0]
    gene_matrix = [syn.get(synid, followLink=True)
                   for filename, synid in consortiumRelease[2]
                   if filename == "data_gene_matrix.txt"][0]

    clinicalDf = pd.read_csv(clinical.path, sep="\t", comment="#")
    gene_matrixdf = pd.read_csv(gene_matrix.path, sep="\t")

    removeForPublicSamples = process_functions.seqDateFilter(
        clinicalDf, processingDate, publicReleaseCutOff)
    # comment back in when public release filter back on
    # publicReleaseSamples = publicReleaseSamples.append(keepForPublicSamples)
    # Make sure all null oncotree codes are removed
    clinicalDf = clinicalDf[~clinicalDf['ONCOTREE_CODE'].isnull()]
    publicReleaseSamples = clinicalDf.SAMPLE_ID[
        ~clinicalDf.SAMPLE_ID.isin(removeForPublicSamples)]

    existing_seq_dates = \
        clinicalDf.SEQ_DATE[clinicalDf.SAMPLE_ID.isin(publicReleaseSamples)]

    logger.info("SEQ_DATES for public release: " +
                ", ".join(set(existing_seq_dates.astype(str))))

    # Clinical release scope filter
    # If consortium -> Don't release to public
    clinicalReleaseScope = syn.tableQuery(
        "SELECT * FROM syn8545211 where releaseScope = 'public'")
    publicRelease = clinicalReleaseScope.asDataFrame()

    allClin = clinicalDf[clinicalDf['SAMPLE_ID'].isin(publicReleaseSamples)]
    allClin.to_csv(clinical_path, sep="\t", index=False)

    gene_matrixdf = \
        gene_matrixdf[gene_matrixdf['SAMPLE_ID'].isin(publicReleaseSamples)]
    gene_matrixdf.to_csv(data_gene_panel_path, sep="\t", index=False)
    storeFile(syn, data_gene_panel_path, public_release_preview, genie_version,
              name="data_gene_matrix.txt")
    storeFile(syn, clinical_path, public_release_preview, genie_version,
              name="data_clinical.txt")

    create_case_lists.main(clinical_path, data_gene_panel_path,
                           database_to_staging.CASE_LIST_PATH, "genie_public")

    caseListFiles = os.listdir(database_to_staging.CASE_LIST_PATH)
    caseListEntities = []
    for casePath in caseListFiles:
        casePath = os.path.join(database_to_staging.CASE_LIST_PATH, casePath)
        caseListEntities.append(
            storeFile(syn, casePath, public_release_preview_caselist,
                      genie_version))

    # Grab mapping table to fill in clinical headers
    mapping_table = syn.tableQuery('SELECT * FROM syn9621600')
    mapping = mapping_table.asDataFrame()
    genePanelEntities = []
    for entName, entId in consortiumRelease[2]:
        # skip files to convert
        if (entName.startswith("data_linear")
                or "meta_" in entName
                or entName.endswith(".html")
                or entName in ["data_clinical_sample.txt",
                               "data_gene_matrix.txt",
                               "data_clinical_patient.txt",
                               "data_guide.pdf",
                               "release_notes.pdf"]):
            # data_gene_matrix was processed above because it had to be
            # used for generating caselists
            continue
        if entName == "data_clinical.txt":
            patientCols = publicRelease['fieldName'][
                publicRelease['level'] == "patient"].tolist()
            sampleCols = ["PATIENT_ID"]
            sampleCols.extend(publicRelease['fieldName'][
                publicRelease['level'] == "sample"].tolist())
            # clinicalDf is defined on line 127
            clinicalDf = \
                clinicalDf[clinicalDf['SAMPLE_ID'].isin(publicReleaseSamples)]

            # Delete columns that are private scope
            # for private in privateRelease:
            #   del clinicalDf[private]
            process_functions.addClinicalHeaders(clinicalDf, mapping,
                                                 patientCols, sampleCols,
                                                 clinical_sample_path,
                                                 clinicl_patient_path)

            storeFile(syn, clinical_sample_path, public_release_preview,
                      genie_version, name="data_clinical_sample.txt")
            storeFile(syn, clinicl_patient_path, public_release_preview,
                      genie_version, name="data_clinical_patient.txt")

        elif "mutation" in entName:
            mutation = syn.get(entId, followLink=True)
            mutationDf = pd.read_csv(mutation.path, sep="\t", comment="#")
            mutationDf = commonVariantFilter(mutationDf)
            mutationDf['FILTER'] = "PASS"
            mutationDf = mutationDf[
                mutationDf['Tumor_Sample_Barcode'].isin(publicReleaseSamples)]
            text = process_functions.removeFloat(mutationDf)
            with open(mutations_path, 'w') as f:
                f.write(text)
            storeFile(syn, mutations_path, public_release_preview,
                      genie_version, name="data_mutations_extended.txt")

        elif "fusion" in entName:
            fusion = syn.get(entId, followLink=True)
            fusionDf = pd.read_csv(fusion.path, sep="\t")
            fusionDf = fusionDf[
                fusionDf['Tumor_Sample_Barcode'].isin(publicReleaseSamples)]
            fusionDf.to_csv(fusions_path, sep="\t", index=False)
            storeFile(
                syn, fusions_path, public_release_preview,
                genie_version, name="data_fusions.txt")
        elif "CNA" in entName:
            cna = syn.get(entId, followLink=True)
            cnaDf = pd.read_csv(cna.path, sep="\t")
            cna_columns = publicReleaseSamples.append(pd.Series("Hugo_Symbol"))
            # parse out the CNA columns to keep
            cnaDf = cnaDf[cnaDf.columns[cnaDf.columns.isin(cna_columns)]]
            text = process_functions.removeFloat(cnaDf)
            text = text.replace(
                "\t\t", "\tNA\t").replace(
                "\t\t", "\tNA\t").replace(
                '\t\n', "\tNA\n")
            with open(cna_path, "w") as cnaFile:
                cnaFile.write(text)
            storeFile(
                syn, cna_path, public_release_preview,
                genie_version, name="data_CNA.txt")
        elif entName.endswith(".seg"):
            seg = syn.get(entId, followLink=True)
            segDf = pd.read_csv(seg.path, sep="\t")
            segDf = segDf[segDf['ID'].isin(publicReleaseSamples)]
            text = process_functions.removeFloat(segDf)
            with open(seg_path, "w") as segFile:
                segFile.write(text)
            storeFile(syn, seg_path, public_release_preview, genie_version,
                      name="genie_public_data_cna_hg19.seg")
        elif entName == "genomic_information.txt":
            bed = syn.get(entId, followLink=True)
            bedDf = pd.read_csv(bed.path, sep="\t")
            bedDf = bedDf[bedDf.SEQ_ASSAY_ID.isin(allClin.SEQ_ASSAY_ID)]
            bedDf.to_csv(combined_bed_path,sep="\t",index=False)
            storeFile(syn, combined_bed_path, public_release_preview,
                      genie_version, name="genomic_information.txt")
        elif entName.startswith("data_gene_panel"):
            genePanel = syn.get(entId, followLink=True)
            # Create new gene panel naming and store
            fileName = os.path.basename(genePanel.path)
            newFileList = fileName.split("_")
            newFileList[-1] = genie_version + ".txt"
            newFileName = "_".join(newFileList)
            genePanelPath = os.path.join(
                database_to_staging.GENIE_RELEASE_DIR, newFileName)
            shutil.copy(genePanel.path, genePanelPath)
            del newFileList[-1]
            entName = "_".join(newFileList)
            entName = entName + ".txt"
            genepanel_ent = storeFile(
                syn, genePanelPath, public_release_preview,
                genie_version, name=entName)
            genePanelEntities.append(genepanel_ent)
        else:
            ent = syn.get(entId, followLink=True, downloadFile=False)
            copiedId = synapseutils.copy(
                syn, ent, public_release_preview,
                version=ent.versionNumber,
                updateExisting=True, setProvenance=None,
                skipCopyAnnotations=True)
            copiedEnt = syn.get(copiedId[ent.id], downloadFile=False)
            # Set version comment
            copiedEnt.versionComment = genie_version
            syn.store(copiedEnt, forceVersion=False)
    return caseListEntities, genePanelEntities


def get_public_to_consortium_synid_mapping(syn, releaseSynId, test=False):
    '''
    Gets the mapping between public version name
    and its Synapse ids (Can probably be replaced with folder view)
    '''
    temp = synapseutils.walk(syn, releaseSynId)
    officialPublic = dict()
    for dirpath, dirnames, filenames in temp:
        release = os.path.basename(dirpath[0])
        # checkRelease = release.split(".")
        final = [i.split("-") for i in release.split(".")]
        checkRelease = []
        for i in final:
            checkRelease.extend(i)
        if test:
            officialPublic['TESTpublic'] = "syn12299959"
        else:
            if len(checkRelease) == 3 and checkRelease[0] != "0":
                if int(checkRelease[1]) > 0:
                    if checkRelease[0] in ['1', '2']:
                        public_release_name = \
                            str(int(checkRelease[0])+1)+".0.0"
                    else:
                        public_release_name = \
                            str(int(checkRelease[0]))+".0-public"
                    officialPublic[public_release_name] = dirpath[1]
    return officialPublic



"""genie package setup"""
import os
from setuptools import setup, find_packages

# figure out the version
about = {}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "genie", "__version__.py")) as f:
    exec(f.read(), about)

# Add readme
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='aacrgenie',
      version=about["__version__"],
      description='Processing and validation for GENIE',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/Sage-Bionetworks/Genie',
      author='Thomas Yu',
      author_email='thomas.yu@sagebionetworks.org',
      license='MIT',
      packages=find_packages(),
      zip_safe=False,
      python_requires='>=3.6',
      entry_points={'console_scripts': ['genie = genie.__main__:main']},
      scripts=['bin/input_to_database.py',
               'bin/database_to_staging.py'],
      install_requires=['pandas>=1.0',
                        'synapseclient>=2.0',
                        'httplib2>=0.11.3',
                        'pycrypto>=2.6.1',
                        'PyYAML>=5.1'])



# Import logging last to not take in synapseclient logging
import logging
from .__version__ import __version__

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
logging.getLogger("keyring").setLevel(logging.WARNING)



from unittest import mock

import pandas as pd
import pytest
import synapseclient

from genie_registry.clinicalSP import clinicalSP

syn = mock.create_autospec(synapseclient.Synapse)

clinsp_class = clinicalSP(syn, "SAGE")


def test_filetype():
    assert clinsp_class._fileType == "clinicalSP"


def test_incorrect_validatefilename():
    with pytest.raises(AssertionError):
        clinsp_class.validateFilename(['foo'])


def test_correct_validatefilename():
    assert clinsp_class.validateFilename(
        ["nonGENIE_data_clinical.txt"]) == "clinicalSP"


def test_perfect__processing():
    expected_clindf = pd.DataFrame(dict(
        PATIENT_ID=["GENIE-SAGE-ID1", "GENIE-SAGE-ID2", "GENIE-SAGE-ID3",
                    "GENIE-SAGE-ID4", "GENIE-SAGE-ID5"],
        SAMPLE_ID=["GENIE-SAGE-ID1-1", "GENIE-SAGE-ID2-1", "GENIE-SAGE-ID3-1",
                   "GENIE-SAGE-ID4-1", "GENIE-SAGE-ID5-1"],
        CENTER=["SAGE", "SAGE", "SAGE", "SAGE", "SAGE"],
        SEQ_ASSAY_ID=["SAGE-TEST", "SAGE-TEST", "SAGE-TEST",
                      "SAGE-TEST", "SAGE-TEST"]))
    # TEST patient processing
    clindf = pd.DataFrame(dict(
        PATIENT_ID=["ID1", "ID2", "ID3", "ID4", "ID5"],
        SAMPLE_ID=["ID1-1", "ID2-1", "ID3-1", "ID4-1", "ID5-1"],
        SEQ_ASSAY_ID=["SAGE-TEST", "SAGE-TEST", "SAGE-TEST",
                      "SAGE-TEST", "SAGE-TEST"]))

    clindf = clinsp_class._process(clindf)
    assert expected_clindf.equals(clindf[expected_clindf.columns])


def test_perfect__validate():
    clindf = pd.DataFrame(dict(
        PATIENT_ID=["ID1", "ID2", "ID3", "ID4", "ID5"],
        SAMPLE_ID=["ID1-1", "ID2-1", "ID3-1", "ID4-1", "ID5-1"],
        SEQ_ASSAY_ID=["SAGE-TEST", "SAGE-TEST", "SAGE-TEST",
                      "SAGE-TEST", "SAGE-TEST"]))

    error, warning = clinsp_class._validate(clindf)
    assert error == ""
    assert warning == ""


def test_missingcols__validate():
    clindf = pd.DataFrame()
    error, warning = clinsp_class._validate(clindf)
    expected_errors = (
        "nonGENIE_data_clinical.txt: File must have SAMPLE_ID column.\n"
        "nonGENIE_data_clinical.txt: File must have SEQ_ASSAY_ID column.\n"
        "nonGENIE_data_clinical.txt: File must have PATIENT_ID column.\n")
    assert error == expected_errors
    assert warning == ""


def test_errors__validate():
    clindf = pd.DataFrame(dict(
        PATIENT_ID=[float('nan'), "ID2", "ID3", "ID4", "ID5"],
        SAMPLE_ID=["ID1-1", float('nan'), "ID3-1", "ID4-1", "ID5-1"],
        SEQ_ASSAY_ID=[float('nan'), "SAGE-TEST", "SAGE-TEST",
                      "SAGE-TEST", "SAGE-TEST"]))

    error, warning = clinsp_class._validate(clindf)

    expected_errors = (
        "nonGENIE_data_clinical.txt: "
        "There can't be any blank values for SAMPLE_ID\n"
        "nonGENIE_data_clinical.txt: "
        "There can't be any blank values for PATIENT_ID\n")
    expected_warnings = (
        "nonGENIE_data_clinical.txt: "
        "Please double check your SEQ_ASSAY_ID columns, "
        "there are empty rows.\n")

    assert error == expected_errors
    assert warning == expected_warnings
